#!/bin/bash

# ******************************************************************************
#  NOTICE OF COPYRIGHT AND OWNERSHIP OF SOFTWARE:
#
#  Copyright © 1997-2020 Flexera Software LLC. All Rights Reserved.
#
#  This computer program is the property of Flexera Software LLC of Schaumburg, Illinois, U.S.A. Any
#  use, copy, publication, distribution, display, modification, or transmission of this computer
#  program in whole or in part in any form or by any means without the prior express written
#  permission of Flexera Software LLC is strictly prohibited.
#
#  Except when expressly provided by Flexera Software LLC in writing, possession of this computer
#  program shall not be construed to confer any license or rights under any of Flexera Software
#  LLC's intellectual property rights, whether by estoppel, implication, or otherwise.
#
#  ALL COPIES OF THIS PROGRAM MUST DISPLAY THIS NOTICE OF COPYRIGHT AND OWNERSHIP IN FULL.
# ******************************************************************************/

# Install functions - source, don't call directly.

export SETUP_PID=$$
JAVA=:
ORACLE_PERF=
FNLS_USER=flexnetls
FNLS_GROUP=flexnetls
PERF_ENABLED=false
GC_ENABLED=false
OVERWRITE_CONFIG=false
RESET_DATABASE=false
PORT_SPEC="#port: 7070"
TLS_ENABLED=false
TLS_PORT_SPEC="port: 1443"
TLS_CERT_SPEC="# keystore-path: path-to-your-keystore"
TLS_CERT_PATH=
TLS_CERT_PASSWORD=
TLS_CERT_PASSWORD_SPEC="# keystore-password: changeit"
LOGGING_SPEC="#logging-threshold: INFO"
LOGGING_STYLE="#loggingStyle: CONTINUOUS"
ACTIVE_HOSTID_SPEC="#active-hostid:"
BACKUP_HOSTID_SPEC="#backup-hostid:"
EXTENDED_SUFFIX_SPEC="#extended-suffix:"
DB_PROPERTIES_SPEC=
ENABLE_EXTENDED_METRICS=false
OPTIONS=

# Check if we're root.
#
function should_be_root() {
    if [[ $EUID -ne 0 ]]; then
		echo "You must be root for this operation."
		exit 1
    fi
}

# systemd in use?
#
function should_be_systemd() {
    boot=$(ps -p 1 -o comm=)
    if [ "$boot" != "systemd" ] ; then
		echo "systemd does not appear to be in use ($boot detected)"
		exit 1
    fi
}

# Check port specification: either a numeric port or [interface].port  (e.g [::1].7070).
#
function assert_portspec() {
	spec_regex='^(\[(.+)\]\.)?([0-9]+)[ \t]*$'
	if [[ ! "$2" =~ $spec_regex ]] ; then
   		echo "$1 expects a port specification - port number or [interface].port."
   		exit 1
	fi
}

# Check that argument is a directory path
#
function assert_isdir() {
	if [ ! -d "$2" ] ; then
   		echo "$1 expects a directory argument."
   		exit 1
	fi
}

# Check that argument is a directory path
#
function assert_isfile() {
	if [ ! -f "$2" ] ; then
   		echo "$1 expects a file path."
   		exit 1
	fi
}


# Check that username is valid
#
function assert_username() {
	if ! id -u "$1" >/dev/null 2>&1 ; then
		echo "$1" is not a recognized user name.
		exit 1
	fi
}

# Check that groupname is valid
#
function assert_groupname() {
	if ! getent group "$1" >/dev/null 2>&1 ; then
		echo "$1" is not a recognized group name.
		exit 1
	fi
}

# Assert that the logging threshold is valid
#
function assert_valid_logging_threshold() {
	validCategories=(ERROR WARN INFO DEBUG LICENSING POLICY)
	found=false
	for item in "${validCategories[@]}" ; do
	     [[ "$item" = "$1" ]] && found=true
	done

	if ! $found ; then
		echo $1 is not a logging threshold
		exit 1
	fi
}

# Assert that the logging style is valid
#
function assert_valid_logging_style() {
	validStyles=(CONTINUOUS CONTINUOUS_UTC DAILY_ROLLOVER DAILY_ROLLOVER_UTC JSON JSON_ROLLOVER)
	found=false
	for item in "${validStyles[@]}" ; do
	     [[ "$item" = "$1" ]] && found=true
	done

	if ! $found ; then
		echo $1 is not a logging threshold
		exit 1
	fi
}

# Determine path to java executable.
#
function java_path() {
	# On most Linux distros, /usr/bin/java is a symlink to the currently active Java

	if type -p java; then
		return
	elif [[ -n "$JAVA_HOME" ]] && [[ -x "$JAVA_HOME/bin/java" ]];  then
		echo "$JAVA_HOME/bin/java"
		return
	elif [[ -n "$JRE_HOME" ]] && [[ -x "$JRE_HOME/bin/java" ]];  then
		echo "$JRE_HOME/bin/java"
		return
	else
		>&2 echo Unable to proceed : java is not installed.
		kill -s TERM $SETUP_PID
	fi
}

# Determine JAVA_HOME if not found in environment, using java on PATH.
# Returns empty string on failure.
#
function java_home() {
    readlink -f "$(command -v java)" | sed -e s%/bin/java$%% -e s%/jre$%%
}

# Determine Java version. The parse is good enough to
# yield '1.8', but probably not totally future-proof.
#
function java_version() {
	$JAVA -version 2>&1 | head -1 | cut -d\" -f 2 | cut -d\. -f 1-2
}

# Check for existence (and readability) of list of files
#
function should_exist() {
	for i in "$@"; do
		if [ ! -r "$i" ] ; then
			echo "Unable to install $i - not present"
			exit 1
		fi
	done
}

# Assert that Java 8 or 11 is in use.
#
function should_be_java_8_or_11() {
	version=$(java_version)
	if [[ "$version" == 1.[[:digit:]] ]] && [ "${version#1.}" == 8 ]; then
		return
	fi

	if [[ "$version" == 11.[[:digit:]] ]]; then
		return
	fi

	echo "Error: Java 8 or 11 is required, but has not been detected."
	exit 1
}

# Extract publisher name from producer-settings.xml
#
function publisher() {
	pub=$(sed -n 's:.*<publisher>\(.*\)</publisher>.*:\1:p' ./producer-settings.xml)
	if [ -z "$pub" ] ; then
		>&2 echo Unable to parse producer-settings.xml
		kill -s TERM $SETUP_PID
	fi
	echo "$pub"
}

# Create service user, and make a member of group $FNLS_GROUP
#
function create_server_user() {
    # Does the designated group exist?
    if ! grep -q -i "^${FNLS_GROUP}:" /etc/group; then
		echo "Creating group $FNLS_GROUP"
		groupadd $FNLS_GROUP
    fi

    if ! id -u $FNLS_USER >/dev/null 2>&1; then
		# Specified service user doesn't exist. Create it.
	    echo "Creating user $FNLS_USER"
	    useradd -s /sbin/nologin -g $FNLS_GROUP $FNLS_USER
    fi
}

# Set appropriate permissions for program dir.
#
function set_install_permissions() {
	chown -R $FNLS_USER:$FNLS_GROUP "$PROGRAM_DIR"
}

# Request a setting (log, tsdir). The jar is responsible for parsing producer-settings.xml.
#
function capture_setting( ) {
	local cmdflag=$1

	if ! "$JAVA" -Dbase.dir="$DATA_DIR" $JVMOPTS -jar flexnetls.jar  $cmdflag ; then
		error "Request for settings failed -- command=$cmdflag"
		kill -s TERM $SETUP_PID
	fi
}

# Request deletion of trusted storage
#
function reset_database() {
	"$JAVA" -Dbase.dir="$DATA_DIR" $JVMOPTS -jar flexnetls.jar --reset-database
}

# Do not replace file if content is different.
#
function safe_replace() {
	if [ -f "$1" ] ; then
		# Are they the same?
		if ! cmp "$2" "$1" >/dev/null 2>&1; then
			# Different content. Overwrite specified?
			if $OVERWRITE_CONFIG ; then
				mv "$2" "$1"
			else
				echo "$1 already exists and has different content. Specify --overwrite to replace."
				echo
				echo "Unified diff of changes:"
				echo
				diff -u "$1" "$2"
				echo
				exit 1
			fi
		fi
	else
		# Doesn't exist, so copy over candidate.
		mv "$2" "$1"
	fi
}

# Generate console.sh
#
function generate_console() {
cat <<EOF > "$PROGRAM_DIR"/console.sh
#!/bin/bash

# Run flexnetls in console session. Membership of the ${FNLS_GROUP} group is
# required in order to get write permission on trusted storage.
if ! grep -q -i "^${FNLS_GROUP}:" /etc/group; then
	echo "Required group $FNLS_GROUP does not exist."
	exit 1
fi

if [[ "\$EUID" -ne 0 ]] ; then
	if ! groups |cut -d: -f2-| grep -q -i "${FNLS_GROUP}"; then
		echo "You must be a member of the $FNLS_GROUP group to do this."
		exit 1
	fi
fi

$JAVA -Dbase.dir="$DATA_DIR" $JVMOPTS -jar flexnetls.jar "\$@"
EOF

chown $FNLS_USER:$FNLS_GROUP "$PROGRAM_DIR/console.sh"
chmod +x "$PROGRAM_DIR"/console.sh
chmod g+rx "$PROGRAM_DIR"/console.sh
}


# Generate local-configuration.yaml
#
function generate_local_configuration() {
	CANDIDATE_YAML=$(mktemp /tmp/local-configuration.XXXXXX)

cat <<EOF > "${CANDIDATE_YAML}"
# local-configuration.

# HTTP listening port. Default is 7070. You can bind to an interface with this syntax: '[127.0.0.1].7070'.
$PORT_SPEC

# Specify the hostid to be used. Syntax: 685B3596A5F3/ETHERNET. Has to be one of the hostids seen
# in the /api/1.0/hostids endpoint.
$ACTIVE_HOSTID_SPEC

# Hostid of the backup server, if in failover configuration.
$BACKUP_HOSTID_SPEC

# Extended hostid suffix, if used.
$EXTENDED_SUFFIX_SPEC

# Logging threshold (ERROR,WARN,INFO,DEBUG,POLICY,LICENSING). A value here will override the database.
$LOGGING_SPEC

# Logging style (CONTINUOUS,CONTINUOUS_UTC,DAILY_ROLLOVER, DAILY_ROLLOVER_UTC, JSON_ROLLOVER, JSON) 
$LOGGING_STYLE

$DB_PROPERTIES_SPEC

# JNI Helper Directory, optional. Use if default location is unsuitable (e.g. noexec issues)
#jni-helper-directory:

# HTTPS server mode
https-in:
  # Set to true to enable
  enabled: $TLS_ENABLED
  # HTTPS listening port
  $TLS_PORT_SPEC
  # Path to keystore
  $TLS_CERT_SPEC
  # Keystore password. You can obfuscate this with java -jar flexnetls.jar -password your-password-here
  $TLS_CERT_PASSWORD_SPEC

# HTTPS client mode. You generally don't need to specify this, unless you have an in-house CA chain.
https-out:
  # Set to true to enable
  enabled: false
  # Path to truststore containing server certificate.
  # truststore-path: path-to-your-truststore
  # Truststore password. You can obfuscate this with java -jar flexnetls.jar -password your-password-here
  # truststore-password: changeit
  # Switch off if you're having host validation problems (not recommended)
  host-verify: true
  # Set to true if you're using self-signed certificates (not recommended)
  self-signed: false
EOF

	safe_replace "$PROGRAM_DIR/local-configuration.yaml" "${CANDIDATE_YAML}"
	rm -f "${CANDIDATE_YAML}"

	chown $FNLS_USER:$FNLS_GROUP "$PROGRAM_DIR/local-configuration.yaml"
	chmod g+r "$PROGRAM_DIR/local-configuration.yaml"
}

# Command-line processing
#
function cmdline() {
	OPTS=$(getopt -o hpo -l "help performance gc enable-extended-metrics oracle overwrite program-dir: data-dir: port: tls-port: tls-cert: tls-cert-password: user: group: jre_home: java_home: logging-threshold: logging-style: reset-database" -- "$@")
	if [ $? != 0 ] ; then exit 1 ; fi   	# Error already reported by getopt

	eval set -- "$OPTS"

	while true ; do
	    case "$1" in
		-h|--help)
			usage;
			shift;;
		-o|--oracle)
			# shellcheck disable=SC2034
			ORACLE_PERF="-XX:+UnlockCommercialFeatures -XX:+FlightRecorder"
			shift;;
		-p|--performance)
			# shellcheck disable=SC2034
			PERF_ENABLED=true
			shift;;
		--gc)
			# shellcheck disable=SC2034
			GC_ENABLED=true
			shift;;					
		--enable-extended-metrics)
			ENABLE_EXTENDED_METRICS=true
			shift;;
		--user)
			assert_username "$2"
			FNLS_USER=$2
			shift 2;;
		--group)
			assert_groupname "$2"
			FNLS_GROUP=$2
			shift 2;;
		--program-dir)
			PROGRAM_DIR="$2"
			shift 2;;
		--data-dir)
			DATA_DIR="$2"
			shift 2;;
		--port)
			assert_portspec PORT "$2"
			PORT_SPEC="port: $2"
			shift 2;;
		--tls-port)
			assert_portspec TLS_PORT "$2"
			TLS_PORT_SPEC="port: $2"
			TLS_ENABLED=true
			shift 2;;
		--tls-cert)
			assert_isfile TLS_CERT "$2"
			TLS_CERT_PATH="$2"
			TLS_ENABLED=true
			shift 2;;
		--tls-cert-password)
			TLS_CERT_PASSWORD_SPEC="keystore-password: $2"
			TLS_CERT_PASSWORD="$2"
			TLS_ENABLED=true
			shift 2;;
		--jre_home)
			assert_isdir JRE_HOME "$2"
			JRE_HOME="$2"
			shift 2;;
		--java_home)
			assert_isdir JAVA_HOME "$2"
			JAVA_HOME="$2"
			shift 2;;
		--logging-threshold)
		    assert_valid_logging_threshold "$2"
			LOGGING_SPEC="logging-threshold: $2"
			shift 2;;
		--logging-style)
		    assert_valid_logging_style "$2"
	        LOGGING_STYLE="loggingStyle: $2"
			shift 2;;
		--overwrite)
			OVERWRITE_CONFIG=true
			shift;;
		--reset-database)
			# shellcheck disable=SC2034
			RESET_DATABASE=true
			shift;;
		--) shift; break;;
	    esac
	done

	if [ -n "$*" ] ; then
		echo "Unexpected non-keyword arguments: $*"
		usage
		exit 1
	fi

	if $ENABLE_EXTENDED_METRICS ; then
		EXTENDED_METRICS_SPEC="Environment=\"EXTENDED_METRICS=true\""
	fi
}

# Usage message
#
function standard_usage() {
	echo "$1 configures the FlexNet License Server for $2 service operation."
	echo "   $3"
	echo
	echo "Usage: ./$1 [OPTIONS]"
	echo
	echo "Supported options are:"
	echo
	echo "--program-dir DIR"
	echo "    specify the install location of the service (default $PROGRAM_DIR)."
	echo "--data-dir DIR"
	echo "    specify the location of trusted storage (default $DATA_DIR)."
	echo "--port PORT"
	echo "    specify the HTTP listening port for the service (default 7070)."
	echo "--tls-port PORT"
	echo "    specify the HTTPS listening port for the service (default 1443)."
	echo "tls-cert CERT-PATH"
	echo "    path to server certificate to be used"
	echo "--tls-cert-password PASSWORD"
	echo "    password for server certificate"
	echo "--user USER"
	echo "    specify the username that the service runs under (default flexnetls)"
	echo "--group GROUP"
	echo "    specify the group name that the service runs under (default flexnetls)"
	echo "--java_home DIR"
	echo "    specify the JAVA_HOME path to the installed JDK to use"
	echo "--jre_home DIR"
	echo "    specify the JRE_HOME path to the installed JRE to use"
	echo "--overwrite"
	echo "    overwrite any existing service configuration or local-configuration.yaml file"
	echo "--logging-threshold LEVEL"
	echo "    set default logging level for installation (overrides producer-settings.xml)"
	echo "--logging-style STYLE"
	echo "    set logging style for installation (CONTINUOUS,CONTINUOUS_UTC,DAILY_ROLLOVER,DAILY_ROLLOVER_UTC,JSON_ROLLOVER,JSON)"
	echo
	echo
	echo "If you don't specify JAVA_HOME or JRE_HOME the values for the system Java install will be assumed."
	exit 1
}
