#!/bin/bash

# ******************************************************************************
#  NOTICE OF COPYRIGHT AND OWNERSHIP OF SOFTWARE:
#  
#  Copyright © 1997-2020 Flexera Software LLC. All Rights Reserved.
#  
#  This computer program is the property of Flexera Software LLC of Schaumburg, Illinois, U.S.A. Any
#  use, copy, publication, distribution, display, modification, or transmission of this computer
#  program in whole or in part in any form or by any means without the prior express written
#  permission of Flexera Software LLC is strictly prohibited.
#  
#  Except when expressly provided by Flexera Software LLC in writing, possession of this computer
#  program shall not be construed to confer any license or rights under any of Flexera Software
#  LLC's intellectual property rights, whether by estoppel, implication, or otherwise.
#  
#  ALL COPIES OF THIS PROGRAM MUST DISPLAY THIS NOTICE OF COPYRIGHT AND OWNERSHIP IN FULL.
# ******************************************************************************/

# Flexnet License Server installer script - systemd variant.

# Source useful install functions.
. ./install-functions.sh

function usage() {
	standard_usage install-systemd.sh 'systemd init' '(use install-sysvinit.sh for traditional SysV init distros)'
}

should_be_systemd
should_be_root
should_exist ./flexnetls.jar ./producer-settings.xml

PUBLISHER=$(publisher)
PROGRAM_DIR=/opt/flexnetls/$PUBLISHER

DATA_DIR=/var/opt/flexnetls/$PUBLISHER
ANCHOR_DIR=/tmp/flexnetsas/$PUBLISHER

# Process command-line options
cmdline "$@"

# Deduce JAVA_HOME if not specified.
if [ -z "$JAVA_HOME" ]; then
	export JAVA_HOME=$(java_home)
fi

# Find Java executable or quit. Can use JAVA or JAVA_HOME variables to achieve this.
JAVA=$(java_path)

should_be_java_8_or_11

if [ "$DATA_DIR" = "/var/opt/flexnetls/$PUBLISHER" ] ; then
	# No command-line override, so defer to producer-settings value.
	DATA_DIR=$(capture_setting --show-trusted-storage-dir)
fi

LOGGING_DIR=$(capture_setting --show-logging-dir)

create_server_user

# Stop any existing instance.
if [ $(systemctl is-active flexnetls-${PUBLISHER}) == 'active' ] ; then
	echo Stopping a running flexnetls-${PUBLISHER} service.
	systemctl stop flexnetls-${PUBLISHER}.service
	sleep 2
fi

if $RESET_DATABASE ; then
	reset_database
	exit 0
fi

UNIT=/etc/systemd/system/flexnetls-${PUBLISHER}.service
UNITCONFIGDIR=/etc/systemd/system/flexnetls-${PUBLISHER}.service.d
UNITCONFIG=${UNITCONFIGDIR}/flexnetls.conf

BASE_OPTIONS=
JVMOPTS="-server -Xms2g -Xmx2g -XX:CompressedClassSpaceSize=64m -XX:MetaspaceSize=256m \
		 -XX:+UseG1GC -XX:NewRatio=3 -XX:MaxGCPauseMillis=75 \
		 -XX:G1HeapWastePercent=10 -XX:InitiatingHeapOccupancyPercent=75 -XX:+CMSScavengeBeforeRemark \
		 -XX:+IgnoreUnrecognizedVMOptions --add-opens=java.base/java.lang=ALL-UNNAMED --add-opens=java.base/java.lang.invoke=ALL-UNNAMED \
		 -XX:+ScavengeBeforeFullGC -XX:-OmitStackTraceInFastThrow \
		 -Djava.security.egd=file:/dev/./urandom ${CUSTOM_JVMOPTS}"
if $PERF_ENABLED ; then
	BASE_OPTIONS="--performance-metrics"
fi
if $GC_ENABLED ; then
	JVMOPTS="$JVMOPTS -XX:+PrintGCDetails -XX:+PrintGCDateStamps -XX:+PrintTenuringDistribution -Xloggc:${DATA_DIR}/gc.log -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=${DATA_DIR} ${ORACLE_PERF}"
fi

mkdir -p "${ANCHOR_DIR}"
mkdir -p "$PROGRAM_DIR"
mkdir -p "$UNITCONFIGDIR"

cp flexnetls.jar "$PROGRAM_DIR"
cp producer-settings.xml "$PROGRAM_DIR"

if [ -n "$TLS_CERT_PATH" ] ; then
	TLS_CERT_SPEC="keystore-path: $PROGRAM_DIR/$(basename $TLS_CERT_PATH)"
	cp "$TLS_CERT_PATH" "$PROGRAM_DIR"
fi

if [[ -n "$TLS_CERT_PASSWORD" && "$TLS_CERT_PASSWORD" != "OBF:"* ]]; then
	# Obfuscate the password.
	TLS_CERT_PASSWORD=$(java -jar flexnetls.jar --password "$TLS_CERT_PASSWORD" | cut -d ' ' -f3)
	echo Certificate password obfuscated to $TLS_CERT_PASSWORD
	TLS_CERT_PASSWORD_SPEC="keystore-password: $TLS_CERT_PASSWORD"
fi

generate_local_configuration

CANDIDATE_UNITCONFIG=$(mktemp /tmp/unit-configuration.XXXXXX)

cat <<EOF > ${CANDIDATE_UNITCONFIG}
[Service]

Environment="JAVA_HOME=${JAVA_HOME}"
Environment="JRE_HOME=${JRE_HOME}"
Environment="PUBLISHER=${PUBLISHER}"
Environment="DATA_DIR=${DATA_DIR}"
Environment="FNLS_USER=${FNLS_USER}"
Environment="FNLS_GROUP=${FNLS_GROUP}"
Environment="JVMOPTS=$JVMOPTS"
Environment="DEFINES=-Dbase.dir=${DATA_DIR}"
Environment="OPTIONS=${BASE_OPTIONS}"
${EXTENDED_METRICS_SPEC}
EOF

safe_replace "$UNITCONFIG" "${CANDIDATE_UNITCONFIG}"
rm -f "${CANDIDATE_UNITCONFIG}"
chmod 644 "$UNITCONFIG"

cat <<EOF > $UNIT
[Unit]
Description=FlexnetLS Local License Server for ${PUBLISHER}.
After=network.target syslog.target

[Service]
WorkingDirectory=${PROGRAM_DIR}
PermissionsStartOnly=true
ExecStartPre=/bin/mkdir -p \${DATA_DIR} ${LOGGING_DIR}
ExecStartPre=/bin/chown -R \${FNLS_USER}:\${FNLS_GROUP} \${DATA_DIR} ${LOGGING_DIR}
ExecStartPre=/bin/chmod g+w \${DATA_DIR} ${LOGGING_DIR}
ExecStartPre=/bin/chmod g+s \${DATA_DIR} ${LOGGING_DIR}
ExecStart=$JAVA \$JVMOPTS \$DEFINES -jar flexnetls.jar \$OPTIONS --service
ExecStop= $JAVA \$DEFINES -jar flexnetls.jar --service-shutdown --logging-threshold=DEBUG
KillMode=process
Restart=on-failure
RestartSec=60
LimitNOFILE=65536
User=${FNLS_USER}
SuccessExitStatus=143
TimeoutStopSec=60
StandardOutput=journal
SyslogIdentifier=flexnetls-${PUBLISHER}
SyslogLevelPrefix=true

[Install] 
WantedBy=multi-user.target
EOF

generate_console

set_install_permissions

# Reload systemd configuration, start service and mark it enabled so that it starts automatically after reboot.
systemctl daemon-reload 
systemctl start flexnetls-${PUBLISHER}.service
systemctl enable flexnetls-${PUBLISHER}.service
echo Installed.
